@@ -10,6 +10,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -174,6 +175,8 @@ void query_screen_xinerama();
 void query_screen_normal();
 int viewport_sort(const void *a, const void *b);
 int query_current_screen();
+int query_current_screen_xinerama();
+int query_current_screen_normal();
 void viewport_left();
 void viewport_right();
 int pointinrect(int px, int py, int rx, int ry, int rw, int rh);
@@ -718,17 +721,17 @@ void updategridtext(Window win, struct wininfo *info, int apply_clip, int draw)
 
   char label[3] = "AA";
 
-  int col_selected = 0;
+  int row_selected = 0;
   for (col = 0; col < info->grid_cols; col++) {
     label[0] = 'A';
-    col_selected = (appstate.grid_nav && appstate.grid_nav_col == col 
-                    && appstate.grid_nav_state == GRID_NAV_ROW);
-
     for (row = 0; row < info->grid_rows; row++) {
       int rectwidth = te.width + 25;
       int rectheight = te.height + 8;
       int xpos = cell_width * col + x_off + (cell_width / 2);
       int ypos = cell_height * row + y_off + (cell_height / 2);
+
+      row_selected = (appstate.grid_nav && appstate.grid_nav_row == row 
+                      && appstate.grid_nav_state == GRID_NAV_COL);
       //printf("Grid: %c%c\n", label[0], label[1]);
 
       /* If the current column is the one selected by grid nav, use
@@ -746,7 +749,7 @@ void updategridtext(Window win, struct wininfo *info, int apply_clip, int draw)
         pathcopy = cairo_copy_path(canvas_cairo);
         cairo_set_line_width(shape_cairo, 2);
 
-        if (col_selected) {
+        if (row_selected) {
           cairo_set_source_rgb(canvas_cairo, 0, .3, .3);
         } else {
           cairo_set_source_rgb(canvas_cairo, 0, .2, 0);
@@ -757,7 +760,7 @@ void updategridtext(Window win, struct wininfo *info, int apply_clip, int draw)
         cairo_stroke(canvas_cairo);
         cairo_path_destroy(pathcopy);
 
-        if (col_selected) {
+        if (row_selected) {
           cairo_set_source_rgb(canvas_cairo, 1, 1, 1);
         } else {
           cairo_set_source_rgb(canvas_cairo, .8, .8, .8);
@@ -1029,7 +1032,8 @@ void cmd_windowzoom(char *args) {
   Window curwin;
   Window rootwin;
   Window dummy_win;
-  int x, y, width, height, border_width, depth;
+  int x, y;
+  unsigned int width, height, border_width, depth;
 
   xdo_window_get_active(xdo, &curwin);
   XGetGeometry(xdo->xdpy, curwin, &rootwin, &x, &y, &width, &height,
@@ -1147,7 +1151,7 @@ void cmd_grid_nav(char *args) {
     appstate.grid_nav = 0;
   } else {
     appstate.grid_nav = 1;
-    appstate.grid_nav_state = GRID_NAV_COL;
+    appstate.grid_nav_state = GRID_NAV_ROW;
   }
 
   appstate.need_draw = 1;
@@ -1309,8 +1313,16 @@ void update() {
     }
   }
 
+
   if (resize && move) {
+    //printf("=> %ld: %dx%d @ %d,%d\n", zone, wininfo.w, wininfo.h, wininfo.x,
+           //wininfo.y);
     XMoveResizeWindow(dpy, zone, wininfo.x, wininfo.y, wininfo.w, wininfo.h);
+
+    /* Under Gnome3/GnomeShell, it seems to ignore this move+resize request
+     * unless we sync and sleep here. Sigh. Gnome is retarded. */
+    //XSync(dpy, 0);
+    //usleep(5000);
   } else if (resize) {
     XResizeWindow(dpy, zone, wininfo.w, wininfo.h);
   } else if (move) {
@@ -1462,14 +1474,14 @@ handler_info_t handle_recording(XKeyEvent *e) {
     if (rec->keycode == e->keycode) {
       g_ptr_array_free(rec->commands, TRUE);
       g_ptr_array_remove_index_fast(recordings, i);
-      i--; /* array removal will shift everything down one to make up for the loss 
-              we'll need to redo this index */
+      i--; /* array removal will shift everything down one to make up for the
+            * loss we'll need to redo this index */
     }
   }
 
   //printf("Recording as keycode:%d\n", e->keycode);
   active_recording->keycode = e->keycode;
-  return;
+  return HANDLE_CONTINUE;
 }
 
 handler_info_t handle_gridnav(XKeyEvent *e) {
@@ -1510,19 +1522,22 @@ handler_info_t handle_gridnav(XKeyEvent *e) {
     appstate.need_draw = 1;
     update();
     save_history_point();
+
+    /* We have a full set of coordinates now; select that grid position */
+    cell_select(appstate.grid_nav_col, appstate.grid_nav_row);
+    update();
+    save_history_point();
+    appstate.grid_nav_row = -1;
+    appstate.grid_nav_col = -1;
   } else if (appstate.grid_nav_state == GRID_NAV_ROW) {
     if (val >= wininfo.grid_rows) {
       return HANDLE_CONTINUE; /* Invalid key in this grid, pass */
     }
     appstate.grid_nav_row = val;
-
-    /* We have a full set of coordinates now; select that grid position */
-    cell_select(appstate.grid_nav_col, appstate.grid_nav_row);
     appstate.grid_nav_state = GRID_NAV_COL;
+    appstate.need_draw = 1;
     update();
     save_history_point();
-    appstate.grid_nav_row = -1;
-    appstate.grid_nav_col = -1;
   }
   return HANDLE_STOP;
 }
